// this is a very basic template. use it to start writing your own effects.
// if you want effects with lighting start from one of the GouraudXXXX or PhongXXXX effects

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex0 <string uiname="Texture0";>;
sampler Samp0 = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex0);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

texture Tex1 <string uiname="Texture1";>;
sampler Samp1 = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex1);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

float2 scale;
float2 offset;
float lambda;
float4 lumcoeff = (0.299,0.587,0.114,0);

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd0 : TEXCOORD0;
    float2 TexCd1 :TEXCOORD1;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd0 : TEXCOORD0,
    float4 TexCd1 : TEXCOORD1)
{
    //declare output struct
    vs2ps Out;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    Out.TexCd0 = mul(TexCd0, tTex);
    Out.TexCd1 = mul(TexCd1, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    float4 col = tex2D(Samp0, In.TexCd0);
    
   float4 a =tex2D(Samp0, In.TexCd0);
   float4 b =tex2D(Samp1, In.TexCd1);
   float2 x1 = (offset.x,0)  ;
   float2 y1 = (0,offset.y)  ;

   //get the difference
   float4 curdif = b-a;
	//calculate the gradient
   float4 gradx = tex2D(Samp1, In.TexCd1+x1)-tex2D(Samp1, In.TexCd1-x1);
	gradx += tex2D(Samp0, In.TexCd0+x1)-tex2D(Samp0, In.TexCd0+x1);
   float4 grady = tex2D(Samp1, In.TexCd1+y1)-tex2D(Samp1, In.TexCd1-y1);
	grady += tex2D(Samp0, In.TexCd0+y1)-tex2D(Samp0, In.TexCd0-y1);
   float4 gradmag = sqrt((gradx*gradx)+(grady*grady)+(lambda));

       float4 vx = curdif*(gradx/gradmag);
       float vxd = vx.r;//assumes greyscale
       
       //format output for flowrepos, out(-x,+x,-y,+y)
	float2 xout = (max(vxd,0.),abs(min(vxd,0.)))*scale.x;

        float4 vy = curdif*(grady/gradmag);
        float vyd = vy.r;//assumes greyscale


      //format output for flowrepos, out(-x,+x,-y,+y)
       float2 yout = (max(vyd,0.),abs(min(vyd,0.)))*scale.y;

        col= float4(xout.xy,yout.xy);

    return col;
    
    }

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TSimpleShader
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_2_0 VS();
        PixelShader  = compile ps_2_0 PS();
    }
}

technique TFixedFunction
{
    pass P0
    {
        //transforms
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //texturing
        Sampler[0] = (Samp0);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere
        
        Lighting       = FALSE;

        //shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
